<?php
/**
 * Set and Get the Plugin Data
 *
 * @package                 iCopyDoc Plugins (v1, core 16-08-2023)
 * @subpackage              YML for Yandex Market
 * @since                   4.1.12
 * 
 * @version                 4.2.5 (05-03-2024)
 * @author                  Maxim Glazunov
 * @link                    https://icopydoc.ru/
 * @see                     
 * 
 * @param       array       $data_arr - Optional
 *
 * @depends                 classes:    
 *                          traits:     
 *                          methods:    
 *                          functions:  
 *                          constants:  
 *                          options:    
 */
defined( 'ABSPATH' ) || exit;

class Y4YM_Rules_List {
	/**
	 * The rules of feeds array
	 *
	 * @var array
	 */
	private $rules_arr = [];

	/**
	 * Summary of __construct
	 * 
	 * @param array $rules_arr - Optional
	 */
	public function __construct( $rules_arr = [] ) {
		if ( empty( $rules_arr ) ) {
			$this->rules_arr = [ 
				'yandex_direct' => [ 
					'offer_tag', 'url', 'categoryid', 'picture', 'store', 'pickup', 'delivery', 'name', 'vendor', 'vendorcode', 
					'description', 'video', 'sales_notes', 'manufacturer_warranty', 'country_of_origin', 'age', 'downloadable', 
					'params', 'delivery_options', 'pickup_options' // TODO: добавить 'adult', 
				],
				'yandex_direct_free_from' => [ 
					'offer_tag', 'url', 'categoryid', 'picture', 'store', 'pickup', 'delivery', 'type_prefix', 'vendor', 
					'model', 'vendorcode', 'description', 'video', 'sales_notes', 'manufacturer_warranty', 'country_of_origin', 
					'age', 'downloadable', 'params', 'delivery_options', 'pickup_options' // TODO: добавить 'adult', 
					// ? есть ли в этом формате 'vendorcode'
				],
				'adv' => [ 
					'offer_tag', 'disabled', 'archived', 'params', 'name', 'enable_auto_discounts', 'description', 'picture', 'url',
					'count', 'amount', 'barcode', 'weight', 'dimensions', 'expiry', 'age', 'downloadable', 'sales_notes',
					'manufacturer_warranty', 'vendor', 'model', 'vendorcode', 'store', 'pickup', 'delivery', 'categoryid',
					'vat', 'delivery_options', 'pickup_options', 'condition', 'credit_template', 'supplier', 'min_quantity',
					'step_quantity', 'additional_expenses', 'cofinance_price', 'purchase_price'
				],
				'single_catalog' => [ 
					'offer_tag', 'disabled', 'archived', 'params', 'name', 'enable_auto_discounts', 'description', 'picture', 'url',
					'count', 'barcode', 'weight', 'dimensions', 'expiry', 'period_of_validity_days', 'age', 'downloadable',
					'country_of_origin', 'manufacturer', 'market_sku', 'tn_ved_codes', 'recommend_stock_data',
					'manufacturer_warranty', 'vendor', 'shop_sku', 'vendorcode', 'store', 'pickup', 'delivery', 'categoryid',
					'vat', 'delivery_options', 'pickup_options', 'condition', 'credit_template', 'supplier', 'min_quantity',
					'step_quantity', 'additional_expenses', 'cofinance_price', 'purchase_price'
				],
				'dbs' => [ 
					'offer_tag', 'disabled', 'archived', 'params', 'name', 'enable_auto_discounts', 'description', 'picture', 'url',
					'count', 'amount', 'barcode', 'weight', 'dimensions', 'expiry', 'age', 'downloadable', 'sales_notes',
					'country_of_origin', 'manufacturer_warranty', 'model', 'vendor', 'vendorcode', 'store', 'pickup',
					'delivery', 'categoryid', 'vat', 'cargo_types', 'delivery_options', 'pickup_options', 'condition',
					'credit_template', 'supplier', 'min_quantity', 'step_quantity', 'additional_expenses', 'cofinance_price', 
					'purchase_price'
				],
				'sales_terms' => [ 
					'offer_tag', 'disabled', 'enable_auto_discounts', 'vat', 'delivery', 'pickup', 'delivery_options',
					'pickup_options', 'store', 'count'
				],
				'sbermegamarket' => [ 
					'offer_tag', 'disabled', 'age', 'amount', 'barcode', 'categoryid', 'condition', 'count', 'country_of_origin',
					'credit_template', 'delivery', 'description', 'dimensions', 'downloadable', 'enable_auto_discounts',
					'expiry', 'instock', 'keywords', 'manufacturer_warranty', 'manufacturer', 'market_sku', 'min_quantity',
					'model', 'name', 'params', 'period_of_validity_days', 'pickup_options', 'pickup', 'picture', 'premium_price',
					'recommend_stock_data', 'sales_notes', 'shop_sku', 'step_quantity', 'store', 'supplier', 'tn_ved_codes',
					'url', 'vat', 'vendor', 'vendorcode', 'weight'
				],
				'vk' => [ 
					'offer_tag', 'disabled', 'age', 'barcode', 'categoryid', 'condition', 'count', 'country_of_origin',
					'delivery', 'description', 'dimensions', 'downloadable', 'expiry', 'instock', 'manufacturer_warranty',
					'manufacturer', 'model', 'name', 'params', 'period_of_validity_days', 'pickup_options', 'pickup', 'picture',
					'sales_notes', 'shop_sku', 'store', 'tn_ved_codes', 'url', 'vendor', 'vendorcode', 'weight'
				],
				'all_elements' => [ 
					'offer_tag', 'disabled', 'archived', 'age', 'amount', 'barcode', 'categoryid', 'condition', 'count', 'country_of_origin',
					'credit_template', 'delivery_options', 'delivery', 'description', 'dimensions', 'downloadable',
					'enable_auto_discounts', 'expiry', 'instock', 'keywords', 'manufacturer_warranty', 'manufacturer',
					'market_sku', 'min_quantity', 'model', 'name', 'outlets', 'params', 'period_of_validity_days',
					'pickup_options', 'pickup', 'picture', 'premium_price', 'recommend_stock_data', 'sales_notes',
					'shop_sku', 'step_quantity', 'store', 'supplier', 'tn_ved_codes', 'url', 'vat', 'cargo_types', 'vendor',
					'vendorcode', 'video', 'weight', 'price_rrp', 'additional_expenses', 'cofinance_price', 'purchase_price',
					'type_prefix'
				],
				'ozon' => [ 
					'offer_tag', 'min_price', 'outlets', 'disabled', 'name', 'url', 'premium_price',
					'count', 'amount', 'categoryid', 'market_sku'
				],
				'yandex_webmaster' => [ 
					'offer_tag', 'disabled', 'barcode', 'categoryid', 'condition', 'country_of_origin', 'credit_template',
					'delivery_options', 'delivery', 'description', 'dimensions', 'instock', 'keywords', 'manufacturer',
					'market_sku', 'min_quantity', 'model', 'name', 'params', 'period_of_validity_days', 'pickup_options',
					'pickup', 'picture', 'recommend_stock_data', 'sales_notes', 'shop_sku', 'step_quantity', 'tn_ved_codes',
					'url', 'cargo_types', 'vendor', 'vendorcode', 'weight'
				],
				'products_and_offers' => [ 
					'offer_tag', 'disabled', 'archived', 'barcode', 'categoryid', 'condition', 'credit_template', 'delivery_options',
					'delivery', 'pickup_options', 'description', 'dimensions', 'instock', 'keywords', 'manufacturer',
					'market_sku', 'min_quantity', 'model', 'name', 'params', 'period_of_validity_days',
					'picture', 'recommend_stock_data', 'sales_notes', 'shop_sku', 'step_quantity', 'tn_ved_codes',
					'url', 'cargo_types', 'vendor', 'vendorcode', 'weight'
				]
			];
		} else {
			$this->rules_arr = $rules_arr;
		}

		$this->rules_arr = apply_filters( 'y4ym_f_set_rules_arr', $this->get_rules_arr() );
	}

	/**
	 * Get the rules of feeds array
	 * 
	 * @return array
	 */
	public function get_rules_arr() {
		return $this->rules_arr;
	}
}