<?php
/**
 * Set and Get the Plugin Data
 *
 * @package                 iCopyDoc Plugins (v1, core 16-08-2023)
 * @subpackage              YML for Yandex Market
 * @since                   0.1.0
 * 
 * @version                 4.2.6 (26-03-2024)
 * @author                  Maxim Glazunov
 * @link                    https://icopydoc.ru/
 * @see                     
 * 
 * @param       array       $data_arr - Optional
 *
 * @depends                 classes:    
 *                          traits:     
 *                          methods:    
 *                          functions:  
 *                          constants:  
 *                          options:    
 */
defined( 'ABSPATH' ) || exit;

class Y4YM_Data_Arr {
	/**
	 * The plugin data array
	 *
	 * @var array
	 */
	private $data_arr = [];

	/**
	 * Summary of __construct
	 * 
	 * @param array $data_arr - Optional
	 */
	public function __construct( $data_arr = [] ) {
		if ( empty ( $data_arr ) ) {
			$this->data_arr = [ 
				[ 
					'opt_name' => 'yfym_status_sborki',
					'def_val' => '-1',
					'mark' => 'private',
					'required' => true,
					'type' => 'auto',
					'tab' => 'none'
				],
				[ // дата начала сборки
					'opt_name' => 'yfym_date_sborki',
					'def_val' => '0000000001',
					'mark' => 'private',
					'required' => true,
					'type' => 'auto',
					'tab' => 'none'
				],
				[ // дата завершения сборки
					'opt_name' => 'yfym_date_sborki_end',
					'def_val' => '0000000001',
					'mark' => 'private',
					'required' => true,
					'type' => 'auto',
					'tab' => 'none'
				],
				[  // дата сохранения настроек плагина
					'opt_name' => 'yfym_date_save_set',
					'def_val' => '0000000001',
					'mark' => 'private',
					'required' => true,
					'type' => 'auto',
					'tab' => 'none'
				],
				[  // число товаров, попавших в выгрузку
					'opt_name' => 'yfym_count_products_in_feed',
					'def_val' => '-1',
					'mark' => 'private',
					'required' => true,
					'type' => 'auto',
					'tab' => 'none'
				],
				[ 
					'opt_name' => 'yfym_file_url',
					'def_val' => '',
					'mark' => 'private',
					'required' => true,
					'type' => 'auto',
					'tab' => 'none'
				],
				[ 
					'opt_name' => 'yfym_file_file',
					'def_val' => '',
					'mark' => 'private',
					'required' => true,
					'type' => 'auto',
					'tab' => 'none'
				],
				[ 
					'opt_name' => 'yfym_errors',
					'def_val' => '',
					'mark' => 'private',
					'required' => true,
					'type' => 'auto',
					'tab' => 'none'
				],
				[ 
					'opt_name' => 'yfym_status_cron',
					'def_val' => 'off',
					'mark' => 'private',
					'required' => true,
					'type' => 'auto',
					'tab' => 'none'
				],
				// ------------------- ОСНОВНЫЕ НАСТРОЙКИ -------------------
				[ 
					'opt_name' => 'yfym_run_cron',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => true,
					'type' => 'select',
					'tab' => 'main_tab',
					'data' => [ 
						'label' => __( 'Automatic file creation', 'yml-for-yandex-market' ),
						'desc' => __( 'The refresh interval on your feed', 'yml-for-yandex-market' ),
						'woo_attr' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'hourly', 'text' => __( 'Hourly', 'yml-for-yandex-market' ) ],
							[ 'value' => 'three_hours', 'text' => __( 'Every three hours', 'yml-for-yandex-market' ) ],
							[ 'value' => 'six_hours', 'text' => __( 'Every six hours', 'yml-for-yandex-market' ) ],
							[ 'value' => 'twicedaily', 'text' => __( 'Twice a day', 'yml-for-yandex-market' ) ],
							[ 'value' => 'daily', 'text' => __( 'Daily', 'yml-for-yandex-market' ) ],
							[ 'value' => 'every_two_days', 'text' => __( 'Every two days', 'yml-for-yandex-market' ) ],
							[ 'value' => 'week', 'text' => __( 'Once a week', 'yml-for-yandex-market' ) ]
						]
					]
				],
				[ 
					'opt_name' => 'yfym_ufup',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => true,
					'type' => 'select',
					'tab' => 'main_tab',
					'data' => [ 
						'label' => __( 'Update feed when updating products', 'yml-for-yandex-market' ),
						'desc' => '',
						'woo_attr' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'on', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						]
					]
				],
				[ 
					'opt_name' => 'yfym_feed_assignment',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'main_tab',
					'data' => [ 
						'label' => __( 'Feed assignment', 'yml-for-yandex-market' ),
						'desc' => __( 'Not used in feed. Inner note for your convenience', 'yml-for-yandex-market' ),
						'placeholder' => __( 'For Yandex Market', 'yml-for-yandex-market' ),
						'tr_class' => 'y4ym_tr'
					]
				],
				[ 
					'opt_name' => 'yfym_file_extension',
					'def_val' => 'xml',
					'mark' => 'public',
					'required' => true,
					'type' => 'select',
					'tab' => 'main_tab',
					'data' => [ 
						'label' => __( 'Feed file extension', 'yml-for-yandex-market' ),
						'desc' => '',
						'woo_attr' => false,
						'key_value_arr' => [ 
							[ 'value' => 'xml', 'text' => 'XML (' . __( 'recommend', 'yml-for-yandex-market' ) . ')' ],
							[ 'value' => 'yml', 'text' => 'YML' ]
						]
					]
				],
				[ 
					'opt_name' => 'yfym_feed_name',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'main_tab',
					'data' => [ 
						'label' => __( 'Name of the feed file', 'yml-for-yandex-market' ),
						'desc' => sprintf( '%s. <strong>%s:</strong> %s!',
							__(
								'If you leave the field empty, the default value will be used',
								'yml-for-yandex-market'
							),
							__( 'Important', 'yml-for-yandex-market' ),
							__(
								'Spaces cannot be used',
								'yml-for-yandex-market'
							)
						),
						'placeholder' => 'feed-yml-0',
						'tr_class' => ''
					]
				],
				[ 
					'opt_name' => 'yfym_archive_to_zip',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => true,
					'type' => 'select',
					'tab' => 'main_tab',
					'data' => [ 
						'label' => __( 'Archive to ZIP', 'yml-for-yandex-market' ),
						'desc' => sprintf( '%s: %s',
							__( 'Default', 'yml-for-yandex-market' ),
							__( 'Disabled', 'yml-for-yandex-market' )
						),
						'woo_attr' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						]
					]
				],
				[ 
					'opt_name' => 'yfym_format_date',
					'def_val' => 'rfc_short',
					'mark' => 'public',
					'required' => true,
					'type' => 'select',
					'tab' => 'main_tab',
					'data' => [ 
						'label' => __( 'Format date', 'yml-for-yandex-market' ),
						'desc' => sprintf( '%s: %s',
							__( 'Default', 'yml-for-yandex-market' ),
							'RFC 3339 short'
						),
						'woo_attr' => false,
						'key_value_arr' => [ 
							[ 
								'value' => 'rfc_short',
								'text' => sprintf( '%s (%s)',
									'RFC 3339 short (2022-03-21T17:47)',
									__( 'recommend', 'yml-for-yandex-market' )
								)
							],
							[ 'value' => 'rfc', 'text' => 'RFC 3339 full (2022-03-21T17:47:19+03:00)' ],
							[ 'value' => 'unixtime', 'text' => 'Unix time (2022-03-21 17:47)' ]
						],
						'tr_class' => 'y4ym_tr'
					]
				],
				[ 
					'opt_name' => 'yfym_yml_rules',
					'def_val' => 'yandex_market',
					'mark' => 'public',
					'required' => true,
					'type' => 'select',
					'tab' => 'main_tab',
					'data' => [ 
						'label' => __( 'To follow the rules', 'yml-for-yandex-market' ),
						'desc' => sprintf( '%s <i>(%s)</i>. %s. %s',
							__( 'Exclude products that do not meet the requirements', 'yml-for-yandex-market' ),
							__( 'missing required elements/data', 'yml-for-yandex-market' ),
							__(
								'The plugin will try to automatically remove products from the YML-feed for which the required fields for the feed are not filled',
								'yml-for-yandex-market'
							),
							__( 'Also, this item affects the structure of the file', 'yml-for-yandex-market' )
						),
						'woo_attr' => false,
						'key_value_arr' => [ 
							[ 
								'value' => 'yandex_market',
								'text' => sprintf( '%s ADV (%s)',
									__( 'Yandex Market', 'yml-for-yandex-market' ),
									__( 'Simplified type', 'yml-for-yandex-market' )
								)
							],
							[ 
								'value' => 'yandex_direct',
								'text' => sprintf( '%s (%s)',
									__( 'Yandex Direct', 'yml-for-yandex-market' ),
									__( 'Simplified type', 'yml-for-yandex-market' )
								)
							],
							[ 
								'value' => 'yandex_direct_free_from',
								'text' => sprintf( '%s (%s vendor.model)',
									__( 'Yandex Direct', 'yml-for-yandex-market' ),
									__( 'Free-from type', 'yml-for-yandex-market' )
								)
							],
							[ 
								'value' => 'dbs',
								'text' => sprintf( '%s DBS (%s)',
									__( 'Yandex Market', 'yml-for-yandex-market' ),
									__( 'Simplified type', 'yml-for-yandex-market' )
								)
							],
							[ 
								'value' => 'single_catalog',
								'text' => sprintf( 'FBY, FBY+, FBS (%s)',
									__( 'in a single catalog', 'yml-for-yandex-market' )
								)
							],
							[ 
								'value' => 'sales_terms',
								'text' => sprintf( '%s (%s)',
									__( 'To manage the placement', 'yml-for-yandex-market' ),
									__( 'Yandex Market', 'yml-for-yandex-market' )
								)
							],
							[ 'value' => 'sbermegamarket', 'text' => __( 'SberMegaMarket', 'yml-for-yandex-market' ) ],
							[ 'value' => 'beru', 'text' => __( 'Former Beru', 'yml-for-yandex-market' ) ],
							[ 
								'value' => 'products_and_offers',
								'text' => sprintf( '%s (%s)',
									__( 'Yandex Webmaster', 'yml-for-yandex-market' ),
									__( 'Products and offers', 'yml-for-yandex-market' )
								)
							],
							[ 
								'value' => 'yandex_webmaster',
								'text' => sprintf( '%s (turbo) (%s)',
									__( 'Yandex Webmaster', 'yml-for-yandex-market' ),
									__( 'abolished by Yandex', 'yml-for-yandex-market' )
								)
							],
							[ 
								'value' => 'all_elements',
								'text' => sprintf( '%s (%s)',
									__( 'No rules', 'yml-for-yandex-market' ),
									__( 'Not recommended', 'yml-for-yandex-market' )
								)
							],
							[ 'value' => 'ozon', 'text' => 'OZON' ],
							[ 'value' => 'vk', 'text' => 'ВКонтакте (vk.com)' ]
						],
						'tr_class' => 'y4ym_tr'
					]
				],
				[ 
					'opt_name' => 'yfym_step_export',
					'def_val' => '500',
					'mark' => 'public',
					'required' => true,
					'type' => 'select',
					'tab' => 'main_tab',
					'data' => [ 
						'label' => __( 'Step export', 'yml-for-yandex-market' ),
						'desc' =>
							sprintf( '%s. %s. %s',
								__( 'The value affects the speed of file creation', 'yml-for-yandex-market' ),
								__(
									'If you have any problems with the generation of the file - try to reduce the value in this field',
									'yml-for-yandex-market'
								),
								__( 'More than 500 can only be installed on powerful servers', 'yml-for-yandex-market' )
							),
						'woo_attr' => false,
						'key_value_arr' => [ 
							[ 'value' => '80', 'text' => '80' ],
							[ 'value' => '200', 'text' => '200' ],
							[ 'value' => '300', 'text' => '300' ],
							[ 'value' => '450', 'text' => '450' ],
							[ 'value' => '500', 'text' => '500' ],
							[ 'value' => '800', 'text' => '800' ],
							[ 'value' => '1000', 'text' => '1000' ]
						],
						'tr_class' => 'y4ym_tr'
					]
				],
				[ 
					'opt_name' => 'yfym_cache',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => true,
					'type' => 'select',
					'tab' => 'main_tab',
					'data' => [ 
						'label' => __( 'Ignore plugin cache', 'yml-for-yandex-market' ),
						'desc' => sprintf( '%s: <a 
						href="https://icopydoc.ru/pochemu-ne-obnovilis-tseny-v-fide-para-slov-o-tihih-pravkah/%s">%s</a>',
							__(
								"Changing this option can be useful if your feed prices don't change after syncing",
								'yml-for-yandex-market'
							),
							'?utm_source=yml-for-yandex-market&utm_medium=organic&utm_campaign=in-plugin-yml-for-yandex-market&utm_content=settings&utm_term=about-cache',
							__( 'Learn More', 'yml-for-yandex-market' )
						),
						'woo_attr' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						],
						'tr_class' => 'y4ym_tr'
					]
				],
				[ 
					'opt_name' => 'yfym_do_cash_file',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => true,
					'type' => 'select',
					'tab' => 'main_tab',
					'data' => [ 
						'label' => __(
							'Disable the creation of cache files when saving products',
							'yml-for-yandex-market'
						),
						'desc' => sprintf( '%s. %s',
							__(
								'This option allows you to reduce the load on the site at the time of saving the product card',
								'yml-for-yandex-market'
							),
							__(
								'However, disabling this option leads to a significant increase in the feed creation time',
								'yml-for-yandex-market'
							)
						),
						'woo_attr' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						],
						'tr_class' => ''
					]
				],
				[ 
					'opt_name' => 'yfym_del_identical_ids',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => true,
					'type' => 'select',
					'tab' => 'main_tab',
					'data' => [ 
						'label' => __(
							'Take steps to remove products with the same ID from the feed',
							'yml-for-yandex-market'
						),
						'desc' => sprintf( '%s. %s',
							__(
								'This is an experimental feature',
								'yml-for-yandex-market'
							),
							__(
								'It should only be used if you have an error related to the presence of products with the same identifier in the product feed',
								'yml-for-yandex-market'
							)
						),
						'woo_attr' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						],
						'tr_class' => ''
					]
				],
				[ //  ? удалить целиком					 
					'opt_name' => 'yfym_file_ids_in_yml',
					'def_val' => '',
					'mark' => 'private',
					'required' => false,
					'type' => 'text',
					'tab' => 'none',
					'data' => []
				],
				[ //  ? удалить целиком					 
					'opt_name' => 'yfym_file_ids_in_xml',
					'def_val' => '',
					'mark' => 'private',
					'required' => false,
					'type' => 'text',
					'tab' => 'none',
					'data' => []
				],
				[ //  ? удалить целиком
					'opt_name' => 'yfym_main_product',
					'def_val' => '',
					'mark' => 'private',
					'required' => false,
					'type' => 'text',
					'tab' => 'none',
					'data' => []
				],
				// ------------------- ДАННЫЕ МАГАЗИНА -------------------
				[ 
					'opt_name' => 'yfym_shop_name',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'shop_data_tab',
					'data' => [ 
						'label' => __( 'Shop name', 'yml-for-yandex-market' ),
						'desc' => __(
							'The short name of the store should not exceed 20 characters',
							'yml-for-yandex-market'
						),
						'placeholder' => 'Super Shop',
						'rules' => [ 
							'yandex_market', 'dbs', 'single_catalog', 'sales_terms', 'sbermegamarket', 'beru',
							'products_and_offers', 'yandex_webmaster', 'all_elements', 'ozon', 'vk'
						],
						'tag_name' => 'name'
					]
				],
				[ 
					'opt_name' => 'yfym_company_name',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'shop_data_tab',
					'data' => [ 
						'label' => __( 'Company name', 'yml-for-yandex-market' ),
						'desc' => __( 'Full name of the company that owns the store', 'yml-for-yandex-market' ),
						'placeholder' => 'OOO Top Market',
						'rules' => [ 
							'yandex_market', 'dbs', 'single_catalog', 'sales_terms', 'sbermegamarket', 'beru',
							'products_and_offers', 'yandex_webmaster', 'all_elements', 'ozon', 'vk'
						],
						'tag_name' => 'company'
					]
				],
				[ 
					'opt_name' => 'yfym_warehouse',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'shop_data_tab',
					'data' => [ 
						'label' => __( 'Warehouse', 'yml-for-yandex-market' ) . ' Name/ID',
						'desc' => sprintf( '%s (OZON) %s (%s)',
							__( 'Warehouse name', 'yml-for-yandex-market' ),
							__( 'or ID', 'yml-for-yandex-market' ),
							__( 'SberMegaMarket', 'yml-for-yandex-market' )
						),
						'placeholder' => ''
					]
				],
				[ 
					'opt_name' => 'yfym_currencies',
					'def_val' => 'enabled',
					'mark' => 'public',
					'required' => true,
					'type' => 'select',
					'tab' => 'shop_data_tab',
					'data' => [ 
						'label' => __( 'Element "currencies"', 'yml-for-yandex-market' ),
						'desc' => '',
						'woo_attr' => false,
						'key_value_arr' => [ 
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ]
						],
						'rules' => [ 
							'yandex_market', 'dbs', 'single_catalog', 'sales_terms', 'sbermegamarket', 'beru',
							'products_and_offers', 'yandex_webmaster', 'all_elements', 'ozon', 'vk'
						],
						'tag_name' => 'currencies'
					]
				],
				[ 
					'opt_name' => 'yfym_adult',
					'def_val' => 'no',
					'mark' => 'public',
					'required' => true,
					'type' => 'select',
					'tab' => 'shop_data_tab',
					'data' => [ 
						'label' => __( 'Adult Market', 'yml-for-yandex-market' ),
						'desc' => '',
						'woo_attr' => false,
						'key_value_arr' => [ 
							[ 'value' => 'no', 'text' => __( 'No', 'yml-for-yandex-market' ) ],
							[ 'value' => 'yes', 'text' => __( 'Yes', 'yml-for-yandex-market' ) ]
						],
						'rules' => [ 
							'yandex_market', 'dbs', 'single_catalog', 'sales_terms', 'sbermegamarket', 'beru',
							'products_and_offers', 'yandex_webmaster', 'all_elements', 'ozon', 'vk'
						],
						'tag_name' => 'adult'
					]
				],
				// ------------------- НАСТРОЙКИ АТРИБУТОВ -------------------
				[ 
					'opt_name' => 'yfym_amount',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Quantity of products', 'yml-for-yandex-market' ) . ' (СДЭК)',
						'desc' => __(
							'To make it work you must enable "Manage stock" and indicate "Stock quantity"',
							'yml-for-yandex-market'
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'amount',
						'tag_name' => 'amount'
					]
				],
				[ 
					'opt_name' => 'yfym_shop_sku',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => 'Shop sku',
						'desc' => 'Shop sku',
						'woo_attr' => true,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'products_id', 'text' => __( 'Add from products ID', 'yml-for-yandex-market' ) ],
							[ 'value' => 'sku', 'text' => __( 'Substitute from SKU', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'shop_sku',
						'tag_name' => 'shop-sku'
					]
				],
				[ 
					'opt_name' => 'yfym_count',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Quantity of products', 'yml-for-yandex-market' ),
						'desc' => __(
							'To make it work you must enable "Manage stock" and indicate "Stock quantity"',
							'yml-for-yandex-market'
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'count',
						'tag_name' => 'count'
					]
				],
				[ 
					'opt_name' => 'yfym_auto_disabled',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Automatically remove products from sale', 'yml-for-yandex-market' ),
						'desc' => __( 'Automatically remove products from sale', 'yml-for-yandex-market' ),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'disabled',
						'tag_name' => 'disabled'
					]
				],
				[ 
					'opt_name' => 'yfym_auto_archived',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Automatically transfer products to the archive', 'yml-for-yandex-market' ),
						'desc' => __( 'Automatically transfer products to the archive', 'yml-for-yandex-market' ),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'archived',
						'tag_name' => 'archived'
					]
				],
				[ 
					'opt_name' => 'yfym_market_sku_status',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Add market-sku to feed', 'yml-for-yandex-market' ),
						'desc' => __(
							'Optional when creating a catalog. A must for price recommendations',
							'yml-for-yandex-market'
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'market_sku',
						'tag_name' => 'market-sku'
					]
				],
				[ 
					'opt_name' => 'yfym_manufacturer',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Manufacturer company', 'yml-for-yandex-market' ),
						'desc' => __( 'Manufacturer company', 'yml-for-yandex-market' ),
						'woo_attr' => true,
						'default_value' => true,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'post_meta', 'text' => __( 'Substitute from post meta', 'yml-for-yandex-market' ) ],
							[ 
								'value' => 'default_value',
								'text' => sprintf( '%s "%s"',
									__( 'Default value from field', 'yml-for-yandex-market' ),
									__( 'Default value', 'yml-for-yandex-market' )
								)
							]
						],
						'rules_f_name' => 'manufacturer',
						'tag_name' => 'manufacturer'
					]
				],
				[ 
					'opt_name' => 'yfym_manufacturer_post_meta',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => '',
						'desc' => '',
						'placeholder' => sprintf( '%s / %s',
							__( 'Value', 'yml-for-yandex-market' ),
							__( 'Name post_meta', 'yml-for-yandex-market' )
						),
						'rules_f_name' => 'manufacturer',
					]
				],
				[ 
					'opt_name' => 'yfym_vendor',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Vendor', 'yml-for-yandex-market' ),
						'desc' => __( 'Vendor', 'yml-for-yandex-market' ),
						'woo_attr' => true,
						'default_value' => true,
						// 'brands' => true, // TODO: удалить в след.версиях
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'post_meta', 'text' => __( 'Substitute from post meta', 'yml-for-yandex-market' ) ],
							[ 
								'value' => 'default_value',
								'text' => sprintf( '%s "%s"',
									__( 'Default value from field', 'yml-for-yandex-market' ),
									__( 'Default value', 'yml-for-yandex-market' )
								)
							]
						],
						'rules_f_name' => 'vendor',
						'tag_name' => 'vendor'
					]
				],
				[ 
					'opt_name' => 'yfym_vendor_post_meta',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => '',
						'desc' => '',
						'placeholder' => sprintf( '%s / %s',
							__( 'Value', 'yml-for-yandex-market' ),
							__( 'Name post_meta', 'yml-for-yandex-market' )
						),
						'rules_f_name' => 'vendor'
					]
				],
				[ 
					'opt_name' => 'yfym_country_of_origin',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Country of origin', 'yml-for-yandex-market' ),
						'desc' => __(
							'This element indicates the country where the product was manufactured',
							'yml-for-yandex-market'
						),
						'woo_attr' => true,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'country_of_origin',
						'tag_name' => 'country_of_origin'
					]
				],
				[ 
					'opt_name' => 'yfym_source_id',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Source ID of the product', 'yml-for-yandex-market' ),
						'desc' => '',
						'woo_attr' => true,
						'default_value' => true,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'sku', 'text' => __( 'Substitute from SKU', 'yml-for-yandex-market' ) ],
							[ 'value' => 'post_meta', 'text' => __( 'Substitute from post meta', 'yml-for-yandex-market' ) ],
							[ 
								'value' => 'germanized',
								'text' => __( 'Substitute from', 'yml-for-yandex-market' ) . 'WooCommerce Germanized'
							]
						],
						'rules_f_name' => 'always',
						'tag_name' => ''
					]
				],
				[ 
					'opt_name' => 'yfym_source_id_post_meta',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => '',
						'desc' => '',
						'placeholder' => __( 'Name post_meta', 'yml-for-yandex-market' ),
						'rules_f_name' => 'always'
					]
				],
				[ 
					'opt_name' => 'yfym_group_id',
					'def_val' => 'enabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'The ID of the product model', 'yml-for-yandex-market' ),
						'desc' => '',
						'woo_attr' => true,
						'default_value' => true,
						'key_value_arr' => [ 
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'always',
						'tag_name' => 'group_id'
					]
				],
				[ 
					'opt_name' => 'yfym_on_demand',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Mark products under the order', 'yml-for-yandex-market' ),
						'desc' => __( 'Product under the order', 'yml-for-yandex-market' ),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'always',
						'tag_name' => 'type="on.demand"'
					]
				],
				[ 
					'opt_name' => 'yfym_pickup',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Pickup', 'yml-for-yandex-market' ),
						'desc' => __( 'Option to get order from pickup point', 'yml-for-yandex-market' ),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'true', 'text' => __( 'True', 'yml-for-yandex-market' ) ],
							[ 'value' => 'false', 'text' => __( 'False', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'pickup',
						'tag_name' => 'pickup'
					]
				],
				[ 
					'opt_name' => 'yfym_price_from',
					'def_val' => 'no',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Price from', 'yml-for-yandex-market' ),
						'desc' => sprintf( '%s <strong>from="true"</strong> %s <strong>price</strong><br />
						<strong>%s:</strong><br /><code>&lt;price from=&quot;true&quot;&gt;2000&lt;/price&gt;</code>',
							__( 'Apply the setting Price from', 'yml-for-yandex-market' ),
							__( 'attribute of', 'yml-for-yandex-market' ),
							__( 'Example', 'yml-for-yandex-market' )
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'true', 'text' => __( 'Yes', 'yml-for-yandex-market' ) ],
							[ 'value' => 'false', 'text' => __( 'No', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'always',
						'tag_name' => '...from="true"...'
					]
				],
				[ 
					'opt_name' => 'yfym_oldprice',
					'def_val' => 'no',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Old price', 'yml-for-yandex-market' ),
						'desc' => __(
							'In oldprice indicates the old price of the goods, which must necessarily be higher than the new price (price)',
							'yml-for-yandex-market'
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'yes', 'text' => __( 'Yes', 'yml-for-yandex-market' ) ],
							[ 'value' => 'no', 'text' => __( 'No', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'always',
						'tag_name' => 'oldprice'
					]
				],
				[ 
					'opt_name' => 'yfym_cofinance_price',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Threshold for receiving discounts in Yandex Market', 'yml-for-yandex-market' ),
						'desc' => sprintf( '<a target="_blank" href="%s">%s</a>',
							'//yandex.ru/support/marketplace/marketing/smart-offer.html#sponsored-discounts',
							__( 'Read more on Yandex', 'yml-for-yandex-market' )
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'cofinance_price',
						'tag_name' => 'cofinance_price'
					]
				],
				[ 
					'opt_name' => 'yfym_purchase_price',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Purchase price', 'yml-for-yandex-market' ),
						'desc' => '',
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'purchase_price',
						'tag_name' => 'purchase_price'
					]
				],
				[ 
					'opt_name' => 'yfym_additional_expenses',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Additional costs for the product', 'yml-for-yandex-market' ),
						'desc' => sprintf( '%s. %s',
							__( 'Additional costs for the product', 'yml-for-yandex-market' ),
							__( 'For example, for delivery or packaging', 'yml-for-yandex-market' )
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'additional_expenses',
						'tag_name' => 'additional_expenses'
					]
				],
				[ 
					'opt_name' => 'yfym_delivery',
					'def_val' => 'false',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Delivery', 'yml-for-yandex-market' ),
						'desc' => __(
							'The delivery item must be set to false if the item is prohibited to sell remotely (jewelry, medicines)',
							'yml-for-yandex-market'
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'true', 'text' => __( 'True', 'yml-for-yandex-market' ) ],
							[ 'value' => 'false', 'text' => __( 'False', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'delivery',
						'tag_name' => 'delivery'
					]
				],
				[ 
					'opt_name' => 'yfym_vat',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'VAT rate', 'yml-for-yandex-market' ),
						'desc' => __(
							'This element is used when creating an YML feed for Yandex.Delivery',
							'yml-for-yandex-market'
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'enabled', 'text' => __( 'Enable. No default value', 'yml-for-yandex-market' ) ],
							[ 'value' => 'NO_VAT', 'text' => __( 'No VAT', 'yml-for-yandex-market' ) . ' (NO_VAT)' ],
							[ 'value' => 'VAT_0', 'text' => '0% (VAT_0)' ],
							[ 'value' => 'VAT_10', 'text' => '10% (VAT_10)' ],
							[ 'value' => 'VAT_10_110', 'text' => 'VAT_10_110' ],
							[ 'value' => 'VAT_18', 'text' => '18% (VAT_18)' ],
							[ 'value' => 'VAT_18_118', 'text' => '18/118 (VAT_18_118)' ],
							[ 'value' => 'VAT_20', 'text' => '20% (VAT_20)' ],
							[ 'value' => 'VAT_20_120', 'text' => '20/120 (VAT_20_120)' ]
						],
						'rules_f_name' => 'vat',
						'tag_name' => 'vat'
					]
				],
				[ 
					'opt_name' => 'yfym_video',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Video', 'yml-for-yandex-market' ),
						'desc' => __(
							'This element is used when creating an YML feed for Yandex Direct',
							'yml-for-yandex-market'
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'video',
						'tag_name' => 'video'
					]
				],
				[ 
					'opt_name' => 'yfym_barcode',
					'def_val' => 'no',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Barcode', 'yml-for-yandex-market' ),
						'desc' => '',
						'woo_attr' => true,
						'default_value' => true,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'sku', 'text' => __( 'Substitute from SKU', 'yml-for-yandex-market' ) ],
							[ 'value' => 'post_meta', 'text' => __( 'Substitute from post meta', 'yml-for-yandex-market' ) ],
							[ 
								'value' => 'ean-for-woocommerce',
								'text' => __( 'Substitute from', 'yml-for-yandex-market' ) . ' EAN for WooCommerce'
							],
							[ 
								'value' => 'germanized',
								'text' => __( 'Substitute from', 'yml-for-yandex-market' ) . ' WooCommerce Germanized'
							]
						],
						'rules_f_name' => 'barcode',
						'tag_name' => 'barcode'
					]
				],
				[ 
					'opt_name' => 'yfym_barcode_post_meta',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => '',
						'desc' => '',
						'placeholder' => __( 'Name post_meta', 'yml-for-yandex-market' ),
						'rules_f_name' => 'barcode'
					]
				],
				[ 
					'opt_name' => 'yfym_vendorcode',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Vendor Code', 'yml-for-yandex-market' ),
						'desc' => __( 'Vendor Code', 'yml-for-yandex-market' ),
						'woo_attr' => true,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'sku', 'text' => __( 'Substitute from SKU', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'vendorcode',
						'tag_name' => 'vendorcode'
					]
				],
				[ 
					'opt_name' => 'yfym_cargo_types',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => '«Честный ЗНАК»',
						'desc' => '',
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'cargo_types',
						'tag_name' => 'cargo-types'
					]
				],
				[ 
					'opt_name' => 'yfym_expiry',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Shelf life / service life', 'yml-for-yandex-market' ),
						'desc' => __( 'Shelf life / service life. expiry date / service life', 'yml-for-yandex-market' ),
						'woo_attr' => true,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'expiry',
						'tag_name' => 'expiry'
					]
				],
				[ 
					'opt_name' => 'yfym_period_of_validity_days',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Shelf life', 'yml-for-yandex-market' ),
						'desc' => __( 'Shelf life', 'yml-for-yandex-market' ),
						'woo_attr' => true,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'period_of_validity_days',
						'tag_name' => 'period-of-validity-days'
					]
				],
				[ 
					'opt_name' => 'yfym_downloadable',
					'def_val' => 'off',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Mark downloadable products', 'yml-for-yandex-market' ),
						'desc' => '',
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'off', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'on', 'text' => __( 'On', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'downloadable',
						'tag_name' => 'downloadable'
					]
				],
				[ 
					'opt_name' => 'yfym_age',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Age', 'yml-for-yandex-market' ),
						'desc' => __( 'Age', 'yml-for-yandex-market' ),
						'woo_attr' => true,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'age',
						'tag_name' => 'age'
					]
				],
				[ 
					'opt_name' => 'yfym_type_prefix',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Product type or category', 'yml-for-yandex-market' ),
						'desc' => __( 'This tag is only used in the free-from feed format', 'yml-for-yandex-market' ),
						'woo_attr' => true,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'type_prefix',
						'tag_name' => 'typePrefix'
					]
				],
				[ 
					'opt_name' => 'yfym_model',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Model', 'yml-for-yandex-market' ),
						'desc' => __( 'Model', 'yml-for-yandex-market' ),
						'woo_attr' => true,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'sku', 'text' => __( 'Substitute from SKU', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'model',
						'tag_name' => 'model'
					]
				],
				[ 
					'opt_name' => 'yfym_manufacturer_warranty',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Manufacturer warrant', 'yml-for-yandex-market' ),
						'desc' => __( "This element is used for products that have an official manufacturer's warranty", 'yfym' ) . '.<ul><li>false — ' . __( 'Product does not have an official warranty', 'yml-for-yandex-market' ) . '</li><li>true — ' . __( 'Product has an official warranty', 'yml-for-yandex-market' ) . '</li></ul>',
						'woo_attr' => true,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'alltrue', 'text' => __( 'Add to all', 'yml-for-yandex-market' ) . ' true' ],
							[ 'value' => 'allfalse', 'text' => __( 'Add to all', 'yml-for-yandex-market' ) . ' false' ],
							[ 'value' => 'sku', 'text' => __( 'Substitute from SKU', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'manufacturer_warranty',
						'tag_name' => 'manufacturer_warranty'
					]
				],
				[ 
					'opt_name' => 'yfym_sales_notes_cat',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Sales notes', 'yml-for-yandex-market' ),
						'desc' => __(
							'The text may be up to 50 characters in length. Also in the item is forbidden to specify the terms of delivery and price reduction (discount on merchandise)',
							'yml-for-yandex-market'
						),
						'woo_attr' => true,
						'default_value' => true,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 
								'value' => 'default_value',
								'text' => sprintf( '%s "%s"',
									__( 'Default value from field', 'yml-for-yandex-market' ),
									__( 'Default value', 'yml-for-yandex-market' )
								)
							]
						],
						'rules_f_name' => 'sales_notes',
						'tag_name' => 'sales_notes'
					]
				],
				[ 
					'opt_name' => 'yfym_sales_notes',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => '',
						'desc' => '',
						'placeholder' => __( 'Default value', 'yml-for-yandex-market' ),
						'rules_f_name' => 'sales_notes'
					]
				],
				[ 
					'opt_name' => 'yfym_store',
					'def_val' => 'true',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Store', 'yml-for-yandex-market' ),
						'desc' => sprintf( '<ul><li>%s — %s</li><li>%s — %s</li></ul>',
							__( 'true', 'yml-for-yandex-market' ),
							__( 'The product can be purchased in retail stores', 'yml-for-yandex-market' ),
							__( 'false', 'yml-for-yandex-market' ),
							__( 'the product cannot be purchased in retail stores', 'yml-for-yandex-market' )
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'true', 'text' => __( 'True', 'yml-for-yandex-market' ) ],
							[ 'value' => 'false', 'text' => __( 'False', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'store',
						'tag_name' => 'store'
					]
				],
				[ 
					'opt_name' => 'yfym_condition',
					'def_val' => 'true',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Condition', 'yml-for-yandex-market' ),
						'desc' => sprintf( '%s %s:<br/>%s',
							__( 'Default value', 'yml-for-yandex-market' ),
							__( 'for', 'yml-for-yandex-market' ),
							'(...condition type="X"...)'
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 
								'value' => 'showcasesample',
								'text' => __( 'Showcase sample', 'yml-for-yandex-market' ) . ' (showcasesample)'
							],
							[ 
								'value' => 'reduction',
								'text' => __( 'Reduction', 'yml-for-yandex-market' ) . ' (reduction)'
							],
							[ 
								'value' => 'fashionpreowned',
								'text' => __( 'Fashionpreowned', 'yml-for-yandex-market' ) . ' (fashionpreowned)'
							],
							[ 
								'value' => 'preowned',
								'text' => __( 'Fashionpreowned', 'yml-for-yandex-market' ) . ' (preowned)'
							],
							[ 
								'value' => 'likenew',
								'text' => __( 'Like New', 'yml-for-yandex-market' ) . ' (likenew)'
							]
						],
						'rules_f_name' => 'condition',
						'tag_name' => 'condition'
					]
				],
				[ 
					'opt_name' => 'yfym_reason',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'wp_list_table',
					'data' => [ 
						'default_value' => false,
						'label' => '',
						'desc' => sprintf( '%s %s:<br/>%s [reason]',
							__( 'Default value', 'yml-for-yandex-market' ),
							__( 'for', 'yml-for-yandex-market' ),
							__( 'Reason', 'yml-for-yandex-market' )
						),
						'placeholder' => __( 'Default value', 'yml-for-yandex-market' ),
						'rules_f_name' => 'condition',
						'tag_name' => ''
					]
				],
				[ 
					'opt_name' => 'yfym_quality',
					'def_val' => 'true',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => '',
						'desc' => sprintf( '%s %s:<br/>%s [quality]',
							__( 'Default value', 'yml-for-yandex-market' ),
							__( 'for', 'yml-for-yandex-market' ),
							__( 'Quality', 'yml-for-yandex-market' )
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'perfect', 'text' => __( 'Perfect', 'yml-for-yandex-market' ) ],
							[ 'value' => 'excellent', 'text' => __( 'Excellent', 'yml-for-yandex-market' ) ],
							[ 'value' => 'good', 'text' => __( 'Good', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'condition',
						'tag_name' => ''
					]
				],
				[ 
					'opt_name' => 'yfym_pickup_options',
					'def_val' => 'true',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Pickup of products', 'yml-for-yandex-market' ),
						'desc' => sprintf( '%s pickup-options.<br/><a 
						target="_blank" 
						href="//yandex.ru/support/partnermarket/elements/pickup-options.html#structure">%s</a>',
							__( 'Optional element', 'yml-for-yandex-market' ),
							__( 'Read more on Yandex', 'yml-for-yandex-market' )
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'on', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'pickup_options',
						'tag_name' => 'pickup-options'
					]
				],
				[ 
					'opt_name' => 'yfym_pickup_cost',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'wp_list_table',
					'data' => [ 
						'default_value' => false,
						'label' => '',
						'desc' => sprintf( '%s:<br/><i>%s [order-before] %s pickup-options</i>',
							__( 'Pickup cost', 'yml-for-yandex-market' ),
							__( 'Required element', 'yml-for-yandex-market' ),
							__( 'of attribute', 'yml-for-yandex-market' )
						),
						'placeholder' => '300',
						'rules_f_name' => 'pickup_options',
						'tag_name' => ''
					]
				],
				[ 
					'opt_name' => 'yfym_pickup_days',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'wp_list_table',
					'data' => [ 
						'default_value' => false,
						'label' => '',
						'desc' => sprintf( '%s:<br/><i>%s [order-before] %s pickup-options</i>',
							__( 'Pickup days', 'yml-for-yandex-market' ),
							__( 'Required element', 'yml-for-yandex-market' ),
							__( 'of attribute', 'yml-for-yandex-market' )
						),
						'placeholder' => '4',
						'rules_f_name' => 'pickup_options',
						'tag_name' => ''
					]
				],
				[ 
					'opt_name' => 'yfym_pickup_order_before',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'wp_list_table',
					'data' => [ 
						'default_value' => false,
						'label' => '',
						'desc' => sprintf( '%s:<br/><i>%s [order-before] %s pickup-options</i>',
							__( 'The time in which you need to place an order to get it at this time', 'yml-for-yandex-market' ),
							__( 'Optional element', 'yml-for-yandex-market' ),
							__( 'of attribute', 'yml-for-yandex-market' )
						),
						'placeholder' => '18',
						'rules_f_name' => 'pickup_options',
						'tag_name' => ''
					]
				],
				[ 
					'opt_name' => 'yfym_delivery_options',
					'def_val' => '',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Use delivery-options', 'yml-for-yandex-market' ),
						'desc' => sprintf( '%s delivery-option.<br/><a 
						target="_blank" 
						href="//yandex.ru/support/partnermarket/elements/delivery-options.html#structure">%s</a>',
							__( 'Optional element', 'yml-for-yandex-market' ),
							__( 'Read more on Yandex', 'yml-for-yandex-market' )
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'on', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'delivery_options',
						'tag_name' => 'delivery-option'
					]
				],
				[ 
					'opt_name' => 'yfym_delivery_cost',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'wp_list_table',
					'data' => [ 
						'default_value' => false,
						'label' => '',
						'desc' => sprintf( '%s:<br/><i>%s [cost] %s delivery-option</i>',
							__( 'Delivery cost', 'yml-for-yandex-market' ),
							__( 'Required element', 'yml-for-yandex-market' ),
							__( 'of attribute', 'yml-for-yandex-market' )
						),
						'placeholder' => '300',
						'rules_f_name' => 'delivery_options',
						'tag_name' => ''
					]
				],
				[ 
					'opt_name' => 'yfym_delivery_days',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'wp_list_table',
					'data' => [ 
						'default_value' => false,
						'label' => '',
						'desc' => sprintf( '%s:<br/><i>%s [days] %s delivery-option</i>',
							__( 'Delivery days', 'yml-for-yandex-market' ),
							__( 'Required element', 'yml-for-yandex-market' ),
							__( 'of attribute', 'yml-for-yandex-market' )
						),
						'placeholder' => '2-4',
						'rules_f_name' => 'delivery_options',
						'tag_name' => ''
					]
				],
				[ 
					'opt_name' => 'yfym_order_before',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'wp_list_table',
					'data' => [ 
						'default_value' => false,
						'label' => '',
						'desc' => sprintf( '%s:<br/><i>%s [order-before] %s delivery-option</i>',
							__( 'The time in which you need to place an order to get it at this time', 'yml-for-yandex-market' ),
							__( 'Optional element', 'yml-for-yandex-market' ),
							__( 'of attribute', 'yml-for-yandex-market' )
						),
						'placeholder' => '18',
						'rules_f_name' => 'delivery_options',
						'tag_name' => ''
					]
				],
				[ 
					'opt_name' => 'yfym_delivery_options2',
					'def_val' => '',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => sprintf( '%s<br/><small><i>(%s)</i></small>',
							__( 'Use delivery-options', 'yml-for-yandex-market' ),
							__( 'Add a second delivery methods', 'yml-for-yandex-market' )
						),
						'desc' => sprintf( '%s delivery-option.<br/><a 
						target="_blank" 
						href="//yandex.ru/support/partnermarket/elements/delivery-options.html#structure">%s</a>',
							__( 'Optional element', 'yml-for-yandex-market' ),
							__( 'Read more on Yandex', 'yml-for-yandex-market' )
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'on', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'delivery_options',
						'tag_name' => 'delivery-option'
					]
				],
				[ 
					'opt_name' => 'yfym_delivery_cost2',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'wp_list_table',
					'data' => [ 
						'default_value' => false,
						'label' => '',
						'desc' => sprintf( '%s:<br/><i>%s [cost] %s delivery-option</i>',
							__( 'Delivery cost', 'yml-for-yandex-market' ),
							__( 'Required element', 'yml-for-yandex-market' ),
							__( 'of attribute', 'yml-for-yandex-market' )
						),
						'placeholder' => '500',
						'rules_f_name' => 'delivery_options',
						'tag_name' => ''
					]
				],
				[ 
					'opt_name' => 'yfym_delivery_days2',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'wp_list_table',
					'data' => [ 
						'default_value' => false,
						'label' => '',
						'desc' => sprintf( '%s:<br/><i>%s [days] %s delivery-option</i>',
							__( 'Delivery days', 'yml-for-yandex-market' ),
							__( 'Required element', 'yml-for-yandex-market' ),
							__( 'of attribute', 'yml-for-yandex-market' )
						),
						'placeholder' => '5',
						'rules_f_name' => 'delivery_options',
						'tag_name' => ''
					]
				],
				[ 
					'opt_name' => 'yfym_order_before2',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'wp_list_table',
					'data' => [ 
						'default_value' => false,
						'label' => '',
						'desc' => sprintf( '%s:<br/><i>%s [order-before] %s delivery-option</i>',
							__( 'The time in which you need to place an order to get it at this time', 'yml-for-yandex-market' ),
							__( 'Optional element', 'yml-for-yandex-market' ),
							__( 'of attribute', 'yml-for-yandex-market' )
						),
						'placeholder' => '18',
						'rules_f_name' => 'delivery_options',
						'tag_name' => ''
					]
				],
				[ 
					'opt_name' => 'yfym_params_arr',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Include these attributes in the values Param', 'yml-for-yandex-market' ),
						'desc' => sprintf( '%s: %s',
							__( 'Hint', 'yml-for-yandex-market' ),
							__(
								'To select multiple values, hold down the (ctrl) button on Windows or (cmd) on a Mac. To deselect, press and hold (ctrl) or (cmd), click on the marked items',
								'yml-for-yandex-market'
							)
						),
						'woo_attr' => true,
						'default_value' => false,
						'key_value_arr' => [],
						'multiple' => true,
						'size' => '8',
						'rules_f_name' => 'params',
						'tag_name' => esc_attr( '<param name="ATTR_NAME">ATTR_VAL</param>' )
					]
				],
				[ 
					'opt_name' => 'yfym_behavior_of_params',
					'def_val' => 'default',
					'mark' => 'public',
					'required' => true,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => '',
						'desc' => __( 'If the attribute has multiple values', 'yml-for-yandex-market' ),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 
								'value' => 'default',
								'text' => sprintf( '%s (%s)',
									__( 'Default', 'yml-for-yandex-market' ),
									__( 'No split', 'yml-for-yandex-market' )
								)
							],
							[ 
								'value' => 'split',
								'text' => __( 'Split', 'yml-for-yandex-market' )
							]
						],
						'rules_f_name' => 'params',
						'tag_name' => ''
					]
				],
				[ 
					'opt_name' => 'yfym_ebay_stock',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'wp_list_table',
					'data' => [ 
						'label' => __( 'Add information about stock to feed for EBay', 'yml-for-yandex-market' ),
						'desc' => '',
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						],
						'rules_f_name' => 'params',
						'tag_name' => esc_attr( '<param name="stock">X</param>' )
					]
				],
				// ------------------- ФИЛЬТРАЦИЯ -------------------
				[ 
					'opt_name' => 'yfym_whot_export',
					'def_val' => 'all',
					'mark' => 'public',
					'required' => true,
					'type' => 'select',
					'tab' => 'filtration_tab',
					'data' => [ 
						'label' => __( 'Whot export', 'yml-for-yandex-market' ),
						'desc' => '',
						'woo_attr' => false,
						'key_value_arr' => [ 
							[ 
								'value' => 'all',
								'text' => __( 'Simple & Variable products', 'yml-for-yandex-market' )
							],
							[ 
								'value' => 'simple',
								'text' => __( 'Only simple products', 'yml-for-yandex-market' )
							],
							[ 
								'value' => 'variable',
								'text' => __( 'Only variable products', 'yml-for-yandex-market' )
							]
						]
					]
				],
				[ 
					'opt_name' => 'yfym_desc',
					'def_val' => 'fullexcerpt',
					'mark' => 'public',
					'required' => true,
					'type' => 'select',
					'tab' => 'filtration_tab',
					'data' => [ 
						'label' => __( 'Description of the product', 'yml-for-yandex-market' ),
						'desc' => sprintf( '[description] - %s',
							__( 'The source of the description', 'yml-for-yandex-market' )
						),
						'woo_attr' => false,
						'key_value_arr' => [ 
							[ 
								'value' => 'excerpt',
								'text' => __( 'Only Excerpt description', 'yml-for-yandex-market' )
							],
							[ 
								'value' => 'full',
								'text' => __( 'Only Full description', 'yml-for-yandex-market' )
							],
							[ 
								'value' => 'excerptfull',
								'text' => __( 'Excerpt or Full description', 'yml-for-yandex-market' )
							],
							[ 
								'value' => 'fullexcerpt',
								'text' => __( 'Full or Excerpt description', 'yml-for-yandex-market' )
							],
							[ 
								'value' => 'excerptplusfull',
								'text' => __( 'Excerpt plus Full description', 'yml-for-yandex-market' )
							],
							[ 
								'value' => 'fullplusexcerpt',
								'text' => __( 'Full plus Excerpt description', 'yml-for-yandex-market' )
							]
						],
						'tr_class' => 'y4ym_tr'
					]
				],
				[ 
					'opt_name' => 'yfym_enable_tags_behavior',
					'def_val' => '',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'filtration_tab',
					'data' => [ 
						'label' => __( 'List of allowed tags', 'yml-for-yandex-market' ),
						'desc' => '',
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'default', 'text' => __( 'Default', 'yml-for-yandex-market' ) ],
							[ 'value' => 'custom', 'text' => __( 'From the field below', 'yml-for-yandex-market' ) ]
						]
					]
				],
				[ 
					'opt_name' => 'yfym_enable_tags_custom',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'filtration_tab',
					'data' => [ 
						'default_value' => false,
						'label' => '',
						'desc' => sprintf( '%s <code>p,br,h3</code>',
							__( 'For example', 'yml-for-yandex-market' )
						),
						'placeholder' => 'p,br,h3'
					]
				],
				[ 
					'opt_name' => 'yfym_the_content',
					'def_val' => 'enabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'filtration_tab',
					'data' => [ 
						'label' => __( 'Use the filter', 'yml-for-yandex-market' ) . ' the_content',
						'desc' => sprintf( '%s: %s',
							__( 'Default', 'yml-for-yandex-market' ),
							__( 'Enabled', 'yml-for-yandex-market' )
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						]
					]
				],
				[ 
					'opt_name' => 'yfym_replace_domain',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'text',
					'tab' => 'filtration_tab',
					'data' => [ 
						'default_value' => false,
						'label' => __( 'Change the domain to', 'yml-for-yandex-market' ),
						'desc' => __(
							'The option allows you to change the domain of your site in the feed to any other',
							'yml-for-yandex-market'
						),
						'placeholder' => 'https://site.ru',
						'tr_class' => 'y4ym_tr'
					]
				],
				[ 
					'opt_name' => 'yfym_clear_get',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'filtration_tab',
					'data' => [ 
						'label' => __(
							'Clear URL from GET-paramrs',
							'yml-for-yandex-market'
						),
						'desc' => sprintf( '%s: <a target="_blank" href="https://icopydoc.ru/vklyuchaem-turbo-stranitsy-dlya-magazina-woocommerce-instruktsiya/?utm_source=yml-for-yandex-market&utm_medium=organic&utm_campaign=in-plugin-yml-for-yandex-market&utm_content=settings&utm_term=yandex-turbo-instruction">%s</a>',
							__( 'This option may be useful when setting up Turbo pages', 'yml-for-yandex-market' ),
							__( 'Tips for configuring Turbo pages', 'yml-for-yandex-market' )
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'on', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						]
					]
				],
				[ 
					'opt_name' => 'yfym_behavior_stip_symbol',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'filtration_tab',
					'data' => [ 
						'label' => sprintf( '%s vendorCode %s shop-sku %s',
							__( 'In attributes', 'yml-for-yandex-market' ),
							__( 'and', 'yml-for-yandex-market' ),
							__( 'ampersand', 'yml-for-yandex-market' )
						),
						'desc' => '',
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'default', 'text' => __( 'Default', 'yml-for-yandex-market' ) ],
							[ 'value' => 'del', 'text' => __( 'Delete', 'yml-for-yandex-market' ) ],
							[ 
								'value' => 'slash',
								'text' => __( 'Replace with', 'yml-for-yandex-market' ) . ' /'
							],
							[ 
								'value' => 'amp',
								'text' => __( 'Replace with', 'yml-for-yandex-market' ) . ' amp;'
							]
						]
					]
				],
				[ 
					'opt_name' => 'yfym_var_desc_priority',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'filtration_tab',
					'data' => [ 
						'label' => __(
							'The varition description takes precedence over others',
							'yml-for-yandex-market'
						),
						'desc' => sprintf( '%s: %s',
							__( 'Default', 'yml-for-yandex-market' ),
							__( 'Enabled', 'yml-for-yandex-market' )
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'on', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						]
					]
				],
				[ 
					'opt_name' => 'yfym_no_group_id_arr',
					'def_val' => '', // ? возможно сюда сериализованный массив запихнуть
					'mark' => 'public',
					'required' => true,
					'type' => 'select',
					'tab' => 'filtration_tab',
					'data' => [ 
						'label' => __(
							'Categories of variable products for which group_id is not allowed',
							'yml-for-yandex-market'
						),
						'desc' => __(
							'According to Yandex Market rules in this field you need to mark ALL categories of products not related to "Clothes, Shoes and Accessories", "Furniture", "Cosmetics, perfumes and care", "Baby products", "Accessories for portable electronics". Ie categories for which it is forbidden to use the attribute group_id',
							'yml-for-yandex-market'
						),
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [],
						'multiple' => true,
						'size' => '8',
						'tr_class' => 'y4ym_tr'
					]
				],
				[ 
					'opt_name' => 'yfym_add_in_name_arr',
					'def_val' => '',
					'mark' => 'public',
					'required' => true,
					'type' => 'select',
					'tab' => 'filtration_tab',
					'data' => [ 
						'label' => __( 'Add attributes to the variable products name', 'yml-for-yandex-market' ),
						'desc' => sprintf( '%s. %s',
							__( 'You can only add attributes that are used for variations and that cannot be grouped using', 'yml-for-yandex-market' ),
							__(
								'It works only for variable products that are not in the category "Clothes, Shoes and Accessories", "Furniture", "Cosmetics, perfumes and care", "Baby products", "Accessories for portable electronics"',
								'yml-for-yandex-market'
							)
						),
						'woo_attr' => true,
						'default_value' => false,
						'key_value_arr' => [],
						'multiple' => true,
						'size' => '8'
					]
				],
				[ 
					'opt_name' => 'yfym_separator_type',
					'def_val' => 'type1',
					'mark' => 'public',
					'required' => true,
					'type' => 'select',
					'tab' => 'filtration_tab',
					'data' => [ 
						'label' => __( 'Separator options', 'yml-for-yandex-market' ),
						'desc' => __( 'Separator options', 'yml-for-yandex-market' ),
						'woo_attr' => false,
						'key_value_arr' => [ 
							[ 
								'value' => 'type1',
								'text' => sprintf( '%s 1. (В1:З1, В2:З2, ... Вn:Зn)',
									__( 'Type', 'yml-for-yandex-market' )
								)
							],
							[ 
								'value' => 'type2',
								'text' => sprintf( '%s 2. (В1-З1, В2-З2, ... Вn:Зn)',
									__( 'Type', 'yml-for-yandex-market' )
								)
							],
							[ 
								'value' => 'type3',
								'text' => sprintf( '%s 3. В1:З1, В2:З2, ... Вn:Зn',
									__( 'Type', 'yml-for-yandex-market' )
								)
							],
							[ 
								'value' => 'type4',
								'text' => sprintf( '%s 4. З1 З2 ... Зn',
									__( 'Type', 'yml-for-yandex-market' )
								)
							]
						]
					]
				],
				[ 
					'opt_name' => 'yfym_behavior_onbackorder',
					'def_val' => 'true',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'filtration_tab',
					'data' => [ 
						'label' => __(
							'For pre-order products, establish availability equal to',
							'yml-for-yandex-market'
						),
						'desc' => '',
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'true', 'text' => 'True' ],
							[ 'value' => 'false', 'text' => 'False' ]
						],
						'tr_class' => 'y4ym_tr'
					]
				],
				[ 
					'opt_name' => 'yfym_no_default_png_products',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'filtration_tab',
					'data' => [ 
						'label' => __( 'Remove default.png from YML', 'yml-for-yandex-market' ),
						'desc' => '',
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'on', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						]
					]
				],
				[ 
					'opt_name' => 'yfym_skip_missing_products',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'filtration_tab',
					'data' => [ 
						'label' => sprintf( '%s (%s)',
							__( 'Skip missing products', 'yml-for-yandex-market' ),
							__( 'except for products for which a pre-order is permitted', 'yml-for-yandex-market' )
						),
						'desc' => '',
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'on', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						],
						'tr_class' => 'y4ym_tr'
					]
				],
				[ 
					'opt_name' => 'yfym_skip_backorders_products',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'filtration_tab',
					'data' => [ 
						'label' => __( 'Skip backorders products', 'yml-for-yandex-market' ),
						'desc' => '',
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'on', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						]
					]
				],
				[ 
					'opt_name' => 'yfym_skip_products_without_pic',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'filtration_tab',
					'data' => [ 
						'label' => __( 'Skip products without pictures', 'yml-for-yandex-market' ),
						'desc' => '',
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'on', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						]
					]
				],
				[ 
					'opt_name' => 'yfym_skip_products_without_desc',
					'def_val' => 'disabled',
					'mark' => 'public',
					'required' => false,
					'type' => 'select',
					'tab' => 'filtration_tab',
					'data' => [ 
						'label' => __( 'Skip products without description', 'yml-for-yandex-market' ),
						'desc' => '',
						'woo_attr' => false,
						'default_value' => false,
						'key_value_arr' => [ 
							[ 'value' => 'disabled', 'text' => __( 'Disabled', 'yml-for-yandex-market' ) ],
							[ 'value' => 'enabled', 'text' => __( 'Enabled', 'yml-for-yandex-market' ) ]
						]
					]
				]
			];
		} else {
			$this->data_arr = $data_arr;
		}

		if ( class_exists( 'WOOCS' ) ) {
			global $WOOCS;
			$currencies_arr = $WOOCS->get_currencies();

			if ( is_array( $currencies_arr ) ) {
				$array_keys = array_keys( $currencies_arr );
				for ( $i = 0; $i < count( $array_keys ); $i++ ) {
					$key_value_arr[] = [ 
						'value' => $array_keys[ $i ],
						'text' => $array_keys[ $i ]
					];
				}
			}
			$this->data_arr[] = [ 
				'opt_name' => 'yfym_wooc_currencies',
				'def_val' => '',
				'mark' => 'public',
				'required' => false,
				'type' => 'select',
				'tab' => 'shop_data_tab',
				'data' => [ 
					'label' => __( 'Feed currency', 'yml-for-yandex-market' ),
					'desc' => sprintf( '%s %s. %s.<br/><strong>%s:</strong> %s %s %s',
						__( 'You have plugin installed', 'yml-for-yandex-market' ),
						'WooCommerce Currency Switcher by PluginUs.NET. Woo Multi Currency and Woo Multi Pay',
						__( 'Indicate in what currency the prices should be', 'yml-for-yandex-market' ),
						__( 'Please note', 'yml-for-yandex-market' ),
						__( 'Yandex Market only supports the following currencies', 'yml-for-yandex-market' ),
						'RUR, RUB, UAH, BYN, KZT, UZS, USD, EUR',
						__( 'Choosing a different currency can lead to errors', 'yml-for-yandex-market' )
					),
					'woo_attr' => false,
					'default_value' => false,
					'key_value_arr' => $key_value_arr
				]
			];
		}

		$this->data_arr = apply_filters( 'y4ym_f_set_default_feed_settings_result_arr', $this->get_data_arr() );
	}

	/**
	 * Get the plugin data array
	 * 
	 * @return array
	 */
	public function get_data_arr() {
		return $this->data_arr;
	}

	/**
	 * Get data for tabs
	 * 
	 * @param string $whot
	 * 
	 * @return array	Example: array([0] => opt_key1, [1] => opt_key2, ...)
	 */
	public function get_data_for_tabs( $whot = '' ) {
		$res_arr = [];
		if ( ! empty ( $this->get_data_arr() ) ) {
			// echo get_array_as_string($this->get_data_arr(), '<br/>');
			for ( $i = 0; $i < count( $this->get_data_arr() ); $i++ ) {
				switch ( $whot ) {
					case "main_tab":
					case "shop_data_tab":
					case "tags_settings_tab":
					case "filtration_tab":
						if ( $this->get_data_arr()[ $i ]['tab'] === $whot ) {
							$arr = $this->get_data_arr()[ $i ]['data'];
							$arr['opt_name'] = $this->get_data_arr()[ $i ]['opt_name'];
							$arr['tab'] = $this->get_data_arr()[ $i ]['tab'];
							$arr['type'] = $this->get_data_arr()[ $i ]['type'];
							$res_arr[] = $arr;
						}
						break;
					case "wp_list_table":
						if ( $this->get_data_arr()[ $i ]['tab'] === $whot ) {
							$arr = $this->get_data_arr()[ $i ];
							$res_arr[] = $arr;
						}
						break;
					default:
						if ( $this->get_data_arr()[ $i ]['tab'] === $whot ) {
							$arr = $this->get_data_arr()[ $i ]['data'];
							$arr['opt_name'] = $this->get_data_arr()[ $i ]['opt_name'];
							$arr['tab'] = $this->get_data_arr()[ $i ]['tab'];
							$arr['type'] = $this->get_data_arr()[ $i ]['type'];
							$res_arr[] = $arr;
						}
				}
			}
			// echo get_array_as_string($res_arr, '<br/>');
			return $res_arr;
		} else {
			return $res_arr;
		}
	}

	/**
	 * Get plugin options name
	 * 
	 * @param string $whot
	 * 
	 * @return array	Example: array([0] => opt_key1, [1] => opt_key2, ...)
	 */
	public function get_opts_name( $whot = '' ) {
		$res_arr = [];
		if ( ! empty ( $this->get_data_arr() ) ) {
			for ( $i = 0; $i < count( $this->get_data_arr() ); $i++ ) {
				switch ( $whot ) {
					case "public":
						if ( $this->get_data_arr()[ $i ]['mark'] === 'public' ) {
							$res_arr[] = $this->get_data_arr()[ $i ]['opt_name'];
						}
						break;
					case "private":
						if ( $this->get_data_arr()[ $i ]['mark'] === 'private' ) {
							$res_arr[] = $this->get_data_arr()[ $i ]['opt_name'];
						}
						break;
					default:
						$res_arr[] = $this->get_data_arr()[ $i ]['opt_name'];
				}
			}
			return $res_arr;
		} else {
			return $res_arr;
		}
	}

	/**
	 * Get plugin options name and default date (array)
	 * 
	 * @param string $whot
	 * 
	 * @return array	Example: array(opt_name1 => opt_val1, opt_name2 => opt_val2, ...)
	 */
	public function get_opts_name_and_def_date( $whot = 'all' ) {
		$res_arr = [];
		if ( ! empty ( $this->get_data_arr() ) ) {
			for ( $i = 0; $i < count( $this->get_data_arr() ); $i++ ) {
				switch ( $whot ) {
					case "public":
						if ( $this->get_data_arr()[ $i ]['mark'] === 'public' ) {
							$res_arr[ $this->get_data_arr()[ $i ]['opt_name'] ] = $this->get_data_arr()[ $i ]['def_val'];
						}
						break;
					case "private":
						if ( $this->get_data_arr()[ $i ]['mark'] === 'private' ) {
							$res_arr[ $this->get_data_arr()[ $i ]['opt_name'] ] = $this->get_data_arr()[ $i ]['def_val'];
						}
						break;
					default:
						$res_arr[ $this->get_data_arr()[ $i ]['opt_name'] ] = $this->get_data_arr()[ $i ]['def_val'];
				}
			}
			return $res_arr;
		} else {
			return $res_arr;
		}
	}

	/**
	 * Get plugin options name and default date (stdClass object)
	 * 
	 * @param string $whot
	 * 
	 * @return array<stdClass>
	 */
	public function get_opts_name_and_def_date_obj( $whot = 'all' ) {
		$source_arr = $this->get_opts_name_and_def_date( $whot );

		$res_arr = [];
		foreach ( $source_arr as $key => $value ) {
			$obj = new stdClass();
			$obj->name = $key;
			$obj->opt_def_value = $value;
			$res_arr[] = $obj; // unit obj
			unset( $obj );
		}
		return $res_arr;
	}
}